aboutsummaryrefslogtreecommitdiff
diff options
context:
space:
mode:
authorLorenz Meier <lm@inf.ethz.ch>2012-08-19 11:29:07 +0200
committerLorenz Meier <lm@inf.ethz.ch>2012-08-19 11:29:07 +0200
commit85bc4f683a4a88cc19a35e1147d19ac5b1106019 (patch)
treee8a7faaf7bdc4df0c307e2880f100f2751b1a5c4
parent2a5fcd917428fa6e549214f8066690672b453af0 (diff)
downloadpx4-firmware-85bc4f683a4a88cc19a35e1147d19ac5b1106019.tar.gz
px4-firmware-85bc4f683a4a88cc19a35e1147d19ac5b1106019.tar.bz2
px4-firmware-85bc4f683a4a88cc19a35e1147d19ac5b1106019.zip
Cleaned up position control (WIP), moved PID structs (should become classes) to systemlib, added deamon app example
-rw-r--r--apps/ardrone_control/ardrone_control_helper.c60
-rw-r--r--apps/ardrone_control/ardrone_control_helper.h21
-rw-r--r--apps/examples/px4_deamon_app/Makefile42
-rw-r--r--apps/examples/px4_deamon_app/px4_deamon_app.c130
-rw-r--r--apps/multirotor_control/multirotor_attitude_control.c51
-rw-r--r--apps/multirotor_control/pid.h40
-rw-r--r--apps/multirotor_position_control/Makefile9
-rw-r--r--apps/multirotor_position_control/ardrone_control.c290
-rw-r--r--apps/multirotor_position_control/ardrone_control.h12
-rw-r--r--apps/multirotor_position_control/ardrone_control_helper.c60
-rw-r--r--apps/multirotor_position_control/ardrone_control_helper.h21
-rw-r--r--apps/multirotor_position_control/ardrone_motor_control.c281
-rw-r--r--apps/multirotor_position_control/ardrone_motor_control.h61
-rw-r--r--apps/multirotor_position_control/attitude_control.c459
-rw-r--r--apps/multirotor_position_control/attitude_control.h52
-rw-r--r--apps/multirotor_position_control/multirotor_position_control.c136
-rw-r--r--apps/multirotor_position_control/pid.c109
-rw-r--r--apps/multirotor_position_control/pid.h40
-rw-r--r--apps/multirotor_position_control/position_control.c81
-rw-r--r--apps/multirotor_position_control/position_control.h47
-rw-r--r--apps/multirotor_position_control/rate_control.c320
-rw-r--r--apps/sensors/sensors.c2
-rw-r--r--apps/systemlib/pid/pid.c (renamed from apps/multirotor_control/pid.c)43
-rw-r--r--apps/systemlib/pid/pid.h (renamed from apps/multirotor_position_control/rate_control.h)48
-rw-r--r--nuttx/configs/px4fmu/nsh/appconfig4
25 files changed, 485 insertions, 1934 deletions
diff --git a/apps/ardrone_control/ardrone_control_helper.c b/apps/ardrone_control/ardrone_control_helper.c
deleted file mode 100644
index c073119e0..000000000
--- a/apps/ardrone_control/ardrone_control_helper.c
+++ /dev/null
@@ -1,60 +0,0 @@
-#include "ardrone_control_helper.h"
-#include <unistd.h>
-#include <nuttx/config.h>
-#include <stdio.h>
-#include <stdlib.h>
-
-// int read_sensors_raw(sensors_raw_t *sensors_raw)
-// {
-// static int ret;
-// ret = global_data_wait(&global_data_sensors_raw->access_conf_rate_full);
-
-// if (ret == 0) {
-// memcpy(sensors_raw->gyro_raw, global_data_sensors_raw->gyro_raw, sizeof(sensors_raw->gyro_raw));
-// // printf("Timestamp %d\n", &global_data_sensors_raw->timestamp);
-
-// } else {
-// printf("Controller timeout, no new sensor values available\n");
-// }
-
-// global_data_unlock(&global_data_sensors_raw->access_conf_rate_full);
-// return ret;
-// }
-
-// int read_attitude(global_data_attitude_t *attitude)
-// {
-
-// static int ret;
-// ret = global_data_wait(&global_data_attitude->access_conf);
-
-// if (ret == 0) {
-// memcpy(&attitude->roll, &global_data_attitude->roll, sizeof(global_data_attitude->roll));
-// memcpy(&attitude->pitch, &global_data_attitude->pitch, sizeof(global_data_attitude->pitch));
-// memcpy(&attitude->yaw, &global_data_attitude->yaw, sizeof(global_data_attitude->yaw));
-// memcpy(&attitude->rollspeed, &global_data_attitude->rollspeed, sizeof(global_data_attitude->rollspeed));
-// memcpy(&attitude->pitchspeed, &global_data_attitude->pitchspeed, sizeof(global_data_attitude->pitchspeed));
-// memcpy(&attitude->yawspeed, &global_data_attitude->yawspeed, sizeof(global_data_attitude->yawspeed));
-
-// } else {
-// printf("Controller timeout, no new attitude values available\n");
-// }
-
-// global_data_unlock(&global_data_attitude->access_conf);
-
-
-
-// return ret;
-// }
-
-// void read_quad_motors_setpoint(quad_motors_setpoint_t *rate_setpoint)
-// {
-
-// if (0 == global_data_trylock(&global_data_quad_motors_setpoint->access_conf)) { //TODO: check if trylock is the right choice, maybe only lock?
-// rate_setpoint->motor_front_nw = global_data_quad_motors_setpoint->motor_front_nw;
-// rate_setpoint->motor_right_ne = global_data_quad_motors_setpoint->motor_right_ne;
-// rate_setpoint->motor_back_se = global_data_quad_motors_setpoint->motor_back_se;
-// rate_setpoint->motor_left_sw = global_data_quad_motors_setpoint->motor_left_sw;
-
-// global_data_unlock(&global_data_quad_motors_setpoint->access_conf);
-// }
-// }
diff --git a/apps/ardrone_control/ardrone_control_helper.h b/apps/ardrone_control/ardrone_control_helper.h
deleted file mode 100644
index 22eebe986..000000000
--- a/apps/ardrone_control/ardrone_control_helper.h
+++ /dev/null
@@ -1,21 +0,0 @@
-/*
- * ardrone_control_helper.h
- *
- * Created on: May 15, 2012
- * Author: thomasgubler
- */
-
-#ifndef ARDRONE_CONTROL_HELPER_H_
-#define ARDRONE_CONTROL_HELPER_H_
-
-#include <stdint.h>
-
-// typedef struct {
-// int16_t gyro_raw[3]; // l3gd20
-// } sensors_raw_t;
-
-// /* Copy quad_motors_setpoint values from global memory to private variables */ //TODO: change this once the new mavlink message for rates is available
-// void read_quad_motors_setpoint(quad_motors_setpoint_t *rate_setpoint);
-
-
-#endif /* ARDRONE_CONTROL_HELPER_H_ */
diff --git a/apps/examples/px4_deamon_app/Makefile b/apps/examples/px4_deamon_app/Makefile
new file mode 100644
index 000000000..e4c169872
--- /dev/null
+++ b/apps/examples/px4_deamon_app/Makefile
@@ -0,0 +1,42 @@
+############################################################################
+#
+# Copyright (C) 2012 PX4 Development Team. All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+#
+# 1. Redistributions of source code must retain the above copyright
+# notice, this list of conditions and the following disclaimer.
+# 2. Redistributions in binary form must reproduce the above copyright
+# notice, this list of conditions and the following disclaimer in
+# the documentation and/or other materials provided with the
+# distribution.
+# 3. Neither the name PX4 nor the names of its contributors may be
+# used to endorse or promote products derived from this software
+# without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
+# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
+# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
+#
+############################################################################
+
+#
+# Basic example application
+#
+
+APPNAME = px4_deamon_app
+PRIORITY = SCHED_PRIORITY_DEFAULT
+STACKSIZE = 2048
+
+include $(APPDIR)/mk/app.mk
diff --git a/apps/examples/px4_deamon_app/px4_deamon_app.c b/apps/examples/px4_deamon_app/px4_deamon_app.c
new file mode 100644
index 000000000..e17bf478a
--- /dev/null
+++ b/apps/examples/px4_deamon_app/px4_deamon_app.c
@@ -0,0 +1,130 @@
+/****************************************************************************
+ *
+ * Copyright (C) 2012 PX4 Development Team. All rights reserved.
+ * Author: @author Example User <mail@example.com>
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ * notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in
+ * the documentation and/or other materials provided with the
+ * distribution.
+ * 3. Neither the name PX4 nor the names of its contributors may be
+ * used to endorse or promote products derived from this software
+ * without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
+ * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
+ * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+ * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ ****************************************************************************/
+
+/**
+ * @file px4_deamon_app.c
+ * Deamon application example for PX4 autopilot
+ */
+
+#include <nuttx/config.h>
+#include <unistd.h>
+#include <stdio.h>
+
+static bool thread_should_exit = false; /**< Deamon exit flag */
+static bool thread_running = false; /**< Deamon status flag */
+static int deamon_task; /**< Handle of deamon task / thread */
+
+/**
+ * Deamon management function.
+ */
+__EXPORT int px4_deamon_app_main(int argc, char *argv[]);
+
+/**
+ * Mainloop of deamon.
+ */
+int mavlink_thread_main(int argc, char *argv[]);
+
+/**
+ * Print the correct usage.
+ */
+static void usage(const char *reason);
+
+int mavlink_thread_main(int argc, char *argv[]) {
+
+ printf("[deamon] starting\n");
+
+ while (true) {
+ printf("Hello Deamon!\n");
+ sleep(1);
+ if (thread_should_exit) break;
+ }
+
+ printf("[deamon] exiting.\n");
+
+ return 0;
+}
+
+static void
+usage(const char *reason)
+{
+ if (reason)
+ fprintf(stderr, "%s\n", reason);
+ fprintf(stderr, "usage: deamon {start|stop|status} [-p <additional params>]\n\n");
+ exit(1);
+}
+
+/**
+ * The deamon app only briefly exists to start
+ * the background job. The stack size assigned in the
+ * Makefile does only apply to this management task.
+ *
+ * The actual stack size should be set in the call
+ * to task_create().
+ */
+int px4_deamon_app_main(int argc, char *argv[])
+{
+ if (argc < 1)
+ usage("missing command");
+
+ if (!strcmp(argv[1], "start")) {
+
+ if (thread_running) {
+ printf("deamon already running\n");
+ /* this is not an error */
+ exit(0);
+ }
+
+ thread_should_exit = false;
+ mavlink_task = task_create("deamon", SCHED_PRIORITY_DEFAULT, 4096, deamon_thread_main, (argv) ? (const char **)&argv[2] : (const char **)NULL);
+ thread_running = true;
+ exit(0);
+ }
+
+ if (!strcmp(argv[1], "stop")) {
+ thread_should_exit = true;
+ exit(0);
+ }
+
+ if (!strcmp(argv[1], "status")) {
+ if (thread_running) {
+ printf("\tdeamon app is running\n");
+ } else {
+ printf("\tdeamon app not started\n");
+ }
+ exit(0);
+ }
+
+ usage("unrecognized command");
+ exit(1);
+}
diff --git a/apps/multirotor_control/multirotor_attitude_control.c b/apps/multirotor_control/multirotor_attitude_control.c
index 4391c3957..5d298ab54 100644
--- a/apps/multirotor_control/multirotor_attitude_control.c
+++ b/apps/multirotor_control/multirotor_attitude_control.c
@@ -49,41 +49,40 @@
#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
-#include <px4/attitude_estimator_bm/matrix.h> //TODO: move matrix.h to somewhere else?
#include <float.h>
#include <math.h>
-#include "pid.h"
#include <arch/board/up_hrt.h>
+#include <systemlib/pid/pid.h>
extern int multirotor_write;
extern int gpios;
#define CONTROL_PID_ATTITUDE_INTERVAL 5e-3
-void turn_xy_plane(const float_vect3 *vector, float yaw,
- float_vect3 *result)
-{
- //turn clockwise
- static uint16_t counter;
-
- result->x = (cosf(yaw) * vector->x + sinf(yaw) * vector->y);
- result->y = (-sinf(yaw) * vector->x + cosf(yaw) * vector->y);
- result->z = vector->z; //leave direction normal to xy-plane untouched
-
- counter++;
-}
-
-void navi2body_xy_plane(const float_vect3 *vector, const float yaw,
- float_vect3 *result)
-{
- turn_xy_plane(vector, yaw, result);
-// result->x = vector->x;
-// result->y = vector->y;
-// result->z = vector->z;
- // result->x = cos(yaw) * vector->x + sin(yaw) * vector->y;
- // result->y = -sin(yaw) * vector->x + cos(yaw) * vector->y;
- // result->z = vector->z; //leave direction normal to xy-plane untouched
-}
+// void turn_xy_plane(const float_vect3 *vector, float yaw,
+// float_vect3 *result)
+// {
+// //turn clockwise
+// static uint16_t counter;
+
+// result->x = (cosf(yaw) * vector->x + sinf(yaw) * vector->y);
+// result->y = (-sinf(yaw) * vector->x + cosf(yaw) * vector->y);
+// result->z = vector->z; //leave direction normal to xy-plane untouched
+
+// counter++;
+// }
+
+// void navi2body_xy_plane(const float_vect3 *vector, const float yaw,
+// float_vect3 *result)
+// {
+// turn_xy_plane(vector, yaw, result);
+// // result->x = vector->x;
+// // result->y = vector->y;
+// // result->z = vector->z;
+// // result->x = cos(yaw) * vector->x + sin(yaw) * vector->y;
+// // result->y = -sin(yaw) * vector->x + cos(yaw) * vector->y;
+// // result->z = vector->z; //leave direction normal to xy-plane untouched
+// }
void multirotor_control_attitude(const struct rc_channels_s *rc,
const struct vehicle_attitude_s *att,
diff --git a/apps/multirotor_control/pid.h b/apps/multirotor_control/pid.h
deleted file mode 100644
index a721c839c..000000000
--- a/apps/multirotor_control/pid.h
+++ /dev/null
@@ -1,40 +0,0 @@
-/*
- * pid.h
- *
- * Created on: May 29, 2012
- * Author: thomasgubler
- */
-
-#ifndef PID_H_
-#define PID_H_
-
-#include <stdint.h>
-
-/* PID_MODE_DERIVATIV_CALC calculates discrete derivative from previous error
- * val_dot in pid_calculate() will be ignored */
-#define PID_MODE_DERIVATIV_CALC 0
-/* Use PID_MODE_DERIVATIV_SET if you have the derivative already (Gyros, Kalman) */
-#define PID_MODE_DERIVATIV_SET 1
-
-typedef struct {
- float kp;
- float ki;
- float kd;
- float intmax;
- float sp;
- float integral;
- float error_previous;
- uint8_t mode;
- uint8_t plot_i;
- uint8_t count;
- uint8_t saturated;
-} PID_t;
-
-void pid_init(PID_t *pid, float kp, float ki, float kd, float intmax, uint8_t mode, uint8_t plot_i);
-void pid_set_parameters(PID_t *pid, float kp, float ki, float kd, float intmax);
-//void pid_set(PID_t *pid, float sp);
-float pid_calculate(PID_t *pid, float sp, float val, float val_dot, float dt);
-
-
-
-#endif /* PID_H_ */
diff --git a/apps/multirotor_position_control/Makefile b/apps/multirotor_position_control/Makefile
index 23e2d49fb..f95a4b50a 100644
--- a/apps/multirotor_position_control/Makefile
+++ b/apps/multirotor_position_control/Makefile
@@ -32,14 +32,11 @@
############################################################################
#
-# Makefile to build uORB
+# Makefile to build multirotor position control
#
-APPNAME = ardrone_control
-PRIORITY = SCHED_PRIORITY_MAX - 15
+APPNAME = multirotor_position_control
+PRIORITY = SCHED_PRIORITY_MAX - 25
STACKSIZE = 2048
-# explicit list of sources - not everything is built currently
-CSRCS = ardrone_control.c ardrone_motor_control.c ardrone_control_helper.c rate_control.c attitude_control.c pid.c
-
include $(APPDIR)/mk/app.mk
diff --git a/apps/multirotor_position_control/ardrone_control.c b/apps/multirotor_position_control/ardrone_control.c
deleted file mode 100644
index 89983724e..000000000
--- a/apps/multirotor_position_control/ardrone_control.c
+++ /dev/null
@@ -1,290 +0,0 @@
-/****************************************************************************
- *
- * Copyright (C) 2008-2012 PX4 Development Team. All rights reserved.
- * Author: Lorenz Meier <lm@inf.ethz.ch>
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- * notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- * notice, this list of conditions and the following disclaimer in
- * the documentation and/or other materials provided with the
- * distribution.
- * 3. Neither the name PX4 nor the names of its contributors may be
- * used to endorse or promote products derived from this software
- * without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
- * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
- * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
- * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
- * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
- * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
- ****************************************************************************/
-
-/*
- * @file Implementation of AR.Drone 1.0 / 2.0 control interface
- */
-
-#include <nuttx/config.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <stdbool.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <debug.h>
-#include <termios.h>
-#include <time.h>
-#include <sys/prctl.h>
-#include <arch/board/up_hrt.h>
-#include "ardrone_control.h"
-#include "attitude_control.h"
-#include "rate_control.h"
-#include "ardrone_motor_control.h"
-#include "position_control.h"
-#include <uORB/uORB.h>
-#include <uORB/topics/vehicle_status.h>
-#include <uORB/topics/vehicle_attitude.h>
-#include <uORB/topics/ardrone_control.h>
-#include <uORB/topics/manual_control_setpoint.h>
-#include <uORB/topics/ardrone_motors_setpoint.h>
-#include <uORB/topics/sensor_combined.h>
-
-#include "ardrone_control_helper.h"
-
-__EXPORT int ardrone_control_main(int argc, char *argv[]);
-
-/****************************************************************************
- * Internal Definitions
- ****************************************************************************/
-
-
-enum {
- CONTROL_MODE_RATES = 0,
- CONTROL_MODE_ATTITUDE = 1,
-} control_mode;
-
-/****************************************************************************
- * Private Data
- ****************************************************************************/
-
-/*File descriptors */
-int ardrone_write;
-int gpios;
-
-bool position_control_thread_started;
-
-/****************************************************************************
- * pthread loops
- ****************************************************************************/
-static void *position_control_loop(void *arg)
-{
- struct vehicle_status_s *state = (struct vehicle_status_s *)arg;
- // Set thread name
- prctl(PR_SET_NAME, "ardrone pos ctrl", getpid());
-
- while (1) {
- if (state->state_machine == SYSTEM_STATE_AUTO) {
-// control_position(); //FIXME TODO XXX
- /* temporary 50 Hz execution */
- usleep(20000);
-
- } else {
- position_control_thread_started = false;
- break;
- }
- }
-
- return NULL;
-}
-
-/****************************************************************************
- * main
- ****************************************************************************/
-
-int ardrone_control_main(int argc, char *argv[])
-{
- /* welcome user */
- printf("[ardrone_control] Control started, taking over motors\n");
-
- /* default values for arguments */
- char *ardrone_uart_name = "/dev/ttyS1";
- control_mode = CONTROL_MODE_ATTITUDE;
-
- char *commandline_usage = "\tusage: ardrone_control -d ardrone-devicename -m mode\n\tmodes are:\n\t\trates\n\t\tattitude\n";
-
- /* read commandline arguments */
- int i;
-
- for (i = 1; i < argc; i++) {
- if (strcmp(argv[i], "-d") == 0 || strcmp(argv[i], "--device") == 0) { //ardrone set
- if (argc > i + 1) {
- ardrone_uart_name = argv[i + 1];
-
- } else {
- printf(commandline_usage);
- return 0;
- }
-
- } else if (strcmp(argv[i], "-m") == 0 || strcmp(argv[i], "--mode") == 0) {
- if (argc > i + 1) {
- if (strcmp(argv[i + 1], "rates") == 0) {
- control_mode = CONTROL_MODE_RATES;
-
- } else if (strcmp(argv[i + 1], "attitude") == 0) {
- control_mode = CONTROL_MODE_ATTITUDE;
-
- } else {
- printf(commandline_usage);
- return 0;
- }
-
- } else {
- printf(commandline_usage);
- return 0;
- }
- }
- }
-
- /* open uarts */
- printf("[ardrone_control] AR.Drone UART is %s\n", ardrone_uart_name);
- ardrone_write = open(ardrone_uart_name, O_RDWR | O_NOCTTY | O_NDELAY);
- if (ardrone_write < 0) {
- printf("[ardrone_control] Failed opening AR.Drone UART, exiting.\n");
- exit(ERROR);
- }
-
- /* initialize motors */
- ar_init_motors(ardrone_write, &gpios);
- int counter = 0;
-
- /* Led animation */
- int led_counter = 0;
-
- /* pthread for position control */
- pthread_t position_control_thread;
- position_control_thread_started = false;
-
- /* structures */
- struct vehicle_status_s state;
- struct vehicle_attitude_s att;
- struct ardrone_control_s ar_control;
- struct manual_control_setpoint_s manual;
- struct sensor_combined_s raw;
- struct ardrone_motors_setpoint_s setpoint;
-
- /* subscribe to attitude, motor setpoints and system state */
- int att_sub = orb_subscribe(ORB_ID(vehicle_attitude));
- int state_sub = orb_subscribe(ORB_ID(vehicle_status));
- int manual_sub = orb_subscribe(ORB_ID(manual_control_setpoint));
- int sensor_sub = orb_subscribe(ORB_ID(sensor_combined));
- int setpoint_sub = orb_subscribe(ORB_ID(ardrone_motors_setpoint));
-
- /* publish AR.Drone motor control state */
- int ardrone_pub = orb_advertise(ORB_ID(ardrone_control), &ar_control);
-
- while (1) {
- /* get a local copy of the vehicle state */
- orb_copy(ORB_ID(vehicle_status), state_sub, &state);
- /* get a local copy of manual setpoint */
- orb_copy(ORB_ID(manual_control_setpoint), manual_sub, &manual);
- /* get a local copy of attitude */
- orb_copy(ORB_ID(vehicle_attitude), att_sub, &att);
-
- if (state.state_machine == SYSTEM_STATE_AUTO) {
- if (false == position_control_thread_started) {
- pthread_attr_t position_control_thread_attr;
- pthread_attr_init(&position_control_thread_attr);
- pthread_attr_setstacksize(&position_control_thread_attr, 2048);
- pthread_create(&position_control_thread, &position_control_thread_attr, position_control_loop, &state);
- position_control_thread_started = true;
- }
-
- control_attitude(0, 0, 0, 0, &att, &state, ardrone_pub, &ar_control);
-
- //No check for remote sticks to disarm in auto mode, land/disarm with ground station
-
- } else if (state.state_machine == SYSTEM_STATE_MANUAL) {
- if (control_mode == CONTROL_MODE_RATES) {
- orb_copy(ORB_ID(sensor_combined), sensor_sub, &raw);
- orb_copy(ORB_ID(ardrone_motors_setpoint), setpoint_sub, &setpoint);
- control_rates(&raw, &setpoint);
-
- } else if (control_mode == CONTROL_MODE_ATTITUDE) {
-
- // XXX Add failsafe logic for RC loss situations
- /* hardcore, last-resort safety checking */
- //if (status->rc_signal_lost) {
-
- control_attitude(manual.roll, manual.pitch, manual.yaw,
- manual.throttle, &att, &state, ardrone_pub, &ar_control);
- }
-
- } else {
-
- }
-
- if (counter % 30 == 0) {
- if (led_counter == 0) ar_set_leds(ardrone_write, 0, 1, 0, 0, 0, 0, 0 , 0);
-
- if (led_counter == 1) ar_set_leds(ardrone_write, 1, 1, 0, 0, 0, 0, 0 , 0);
-
- if (led_counter == 2) ar_set_leds(ardrone_write, 1, 0, 0, 0, 0, 0, 0 , 0);
-
- if (led_counter == 3) ar_set_leds(ardrone_write, 0, 0, 0, 1, 0, 0, 0 , 0);
-
- if (led_counter == 4) ar_set_leds(ardrone_write, 0, 0, 1, 1, 0, 0, 0 , 0);
-
- if (led_counter == 5) ar_set_leds(ardrone_write, 0, 0, 1, 0, 0, 0, 0 , 0);
-
- if (led_counter == 6) ar_set_leds(ardrone_write, 0, 0, 0, 0, 0, 1, 0 , 0);
-
- if (led_counter == 7) ar_set_leds(ardrone_write, 0, 0, 0, 0, 1, 1, 0 , 0);
-
- if (led_counter == 8) ar_set_leds(ardrone_write, 0, 0, 0, 0, 1, 0, 0 , 0);
-
- if (led_counter == 9) ar_set_leds(ardrone_write, 0, 0, 0, 0, 0, 0, 0 , 1);
-
- if (led_counter == 10) ar_set_leds(ardrone_write, 0, 0, 0, 0, 0, 0, 1 , 1);
-
- if (led_counter == 11) ar_set_leds(ardrone_write, 0, 0, 0, 0, 0, 0, 1 , 0);
-
- led_counter++;
-
- if (led_counter == 12) led_counter = 0;
- }
-
- /* run at approximately 200 Hz */
- usleep(5000);
-
- // This is a hardcore debug code piece to validate
- // the motor interface
- // uint8_t motorSpeedBuf[5] = {1, 2, 3, 4, 5};
- // ar_get_motor_packet(motorSpeedBuf, 20, 20, 20, 20);
- // write(ardrone_write, motorSpeedBuf, 5);
- // usleep(15000);
-
- counter++;
- }
-
- /* close uarts */
- close(ardrone_write);
- ar_multiplexing_deinit(gpios);
-
- printf("[ardrone_control] ending now...\r\n");
- fflush(stdout);
- return 0;
-}
-
diff --git a/apps/multirotor_position_control/ardrone_control.h b/apps/multirotor_position_control/ardrone_control.h
deleted file mode 100644
index 7f9567f86..000000000
--- a/apps/multirotor_position_control/ardrone_control.h
+++ /dev/null
@@ -1,12 +0,0 @@
-/*
- * ardrone_control.h
- *
- * Created on: Mar 23, 2012
- * Author: thomasgubler
- */
-
-#ifndef ARDRONE_CONTROL_H_
-#define ARDRONE_CONTROL_H_
-
-
-#endif /* ARDRONE_CONTROL_H_ */
diff --git a/apps/multirotor_position_control/ardrone_control_helper.c b/apps/multirotor_position_control/ardrone_control_helper.c
deleted file mode 100644
index c073119e0..000000000
--- a/apps/multirotor_position_control/ardrone_control_helper.c
+++ /dev/null
@@ -1,60 +0,0 @@
-#include "ardrone_control_helper.h"
-#include <unistd.h>
-#include <nuttx/config.h>
-#include <stdio.h>
-#include <stdlib.h>
-
-// int read_sensors_raw(sensors_raw_t *sensors_raw)
-// {
-// static int ret;
-// ret = global_data_wait(&global_data_sensors_raw->access_conf_rate_full);
-
-// if (ret == 0) {
-// memcpy(sensors_raw->gyro_raw, global_data_sensors_raw->gyro_raw, sizeof(sensors_raw->gyro_raw));
-// // printf("Timestamp %d\n", &global_data_sensors_raw->timestamp);
-
-// } else {
-// printf("Controller timeout, no new sensor values available\n");
-// }
-
-// global_data_unlock(&global_data_sensors_raw->access_conf_rate_full);
-// return ret;
-// }
-
-// int read_attitude(global_data_attitude_t *attitude)
-// {
-
-// static int ret;
-// ret = global_data_wait(&global_data_attitude->access_conf);
-
-// if (ret == 0) {
-// memcpy(&attitude->roll, &global_data_attitude->roll, sizeof(global_data_attitude->roll));
-// memcpy(&attitude->pitch, &global_data_attitude->pitch, sizeof(global_data_attitude->pitch));
-// memcpy(&attitude->yaw, &global_data_attitude->yaw, sizeof(global_data_attitude->yaw));
-// memcpy(&attitude->rollspeed, &global_data_attitude->rollspeed, sizeof(global_data_attitude->rollspeed));
-// memcpy(&attitude->pitchspeed, &global_data_attitude->pitchspeed, sizeof(global_data_attitude->pitchspeed));
-// memcpy(&attitude->yawspeed, &global_data_attitude->yawspeed, sizeof(global_data_attitude->yawspeed));
-
-// } else {
-// printf("Controller timeout, no new attitude values available\n");
-// }
-
-// global_data_unlock(&global_data_attitude->access_conf);
-
-
-
-// return ret;
-// }
-
-// void read_quad_motors_setpoint(quad_motors_setpoint_t *rate_setpoint)
-// {
-
-// if (0 == global_data_trylock(&global_data_quad_motors_setpoint->access_conf)) { //TODO: check if trylock is the right choice, maybe only lock?
-// rate_setpoint->motor_front_nw = global_data_quad_motors_setpoint->motor_front_nw;
-// rate_setpoint->motor_right_ne = global_data_quad_motors_setpoint->motor_right_ne;
-// rate_setpoint->motor_back_se = global_data_quad_motors_setpoint->motor_back_se;
-// rate_setpoint->motor_left_sw = global_data_quad_motors_setpoint->motor_left_sw;
-
-// global_data_unlock(&global_data_quad_motors_setpoint->access_conf);
-// }
-// }
diff --git a/apps/multirotor_position_control/ardrone_control_helper.h b/apps/multirotor_position_control/ardrone_control_helper.h
deleted file mode 100644
index 22eebe986..000000000
--- a/apps/multirotor_position_control/ardrone_control_helper.h
+++ /dev/null
@@ -1,21 +0,0 @@
-/*
- * ardrone_control_helper.h
- *
- * Created on: May 15, 2012
- * Author: thomasgubler
- */
-
-#ifndef ARDRONE_CONTROL_HELPER_H_
-#define ARDRONE_CONTROL_HELPER_H_
-
-#include <stdint.h>
-
-// typedef struct {
-// int16_t gyro_raw[3]; // l3gd20
-// } sensors_raw_t;
-
-// /* Copy quad_motors_setpoint values from global memory to private variables */ //TODO: change this once the new mavlink message for rates is available
-// void read_quad_motors_setpoint(quad_motors_setpoint_t *rate_setpoint);
-
-
-#endif /* ARDRONE_CONTROL_HELPER_H_ */
diff --git a/apps/multirotor_position_control/ardrone_motor_control.c b/apps/multirotor_position_control/ardrone_motor_control.c
deleted file mode 100644
index f13427eea..000000000
--- a/apps/multirotor_position_control/ardrone_motor_control.c
+++ /dev/null
@@ -1,281 +0,0 @@
-/****************************************************************************
- *
- * Copyright (C) 2008-2012 PX4 Development Team. All rights reserved.
- * Author: Lorenz Meier <lm@inf.ethz.ch>
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- * notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- * notice, this list of conditions and the following disclaimer in
- * the documentation and/or other materials provided with the
- * distribution.
- * 3. Neither the name PX4 nor the names of its contributors may be
- * used to endorse or promote products derived from this software
- * without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
- * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
- * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
- * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
- * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
- * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
- ****************************************************************************/
-
-/*
- * @file ardrone_motor_control.c
- * Implementation of AR.Drone 1.0 / 2.0 motor control interface
- */
-
-
-
-#include "ardrone_motor_control.h"
-
-static const unsigned long motor_gpios = GPIO_EXT_1 | GPIO_EXT_2 | GPIO_MULTI_1 | GPIO_MULTI_2;
-static const unsigned long motor_gpio[4] = { GPIO_EXT_1, GPIO_EXT_2, GPIO_MULTI_1, GPIO_MULTI_2 };
-
-typedef union {
- uint16_t motor_value;
- uint8_t bytes[2];
-} motor_union_t;
-
-/**
- * @brief Generate the 8-byte motor set packet
- *
- * @return the number of bytes (8)
- */
-void ar_get_motor_packet(uint8_t *motor_buf, uint16_t motor1, uint16_t motor2, uint16_t motor3, uint16_t motor4)
-{
- motor_buf[0] = 0x20;
- motor_buf[1] = 0x00;
- motor_buf[2] = 0x00;
- motor_buf[3] = 0x00;
- motor_buf[4] = 0x00;
- /*
- * {0x20, 0x00, 0x00, 0x00, 0x00};
- * 0x20 is start sign / motor command
- */
- motor_union_t curr_motor;
- uint16_t nineBitMask = 0x1FF;
-
- /* Set motor 1 */
- curr_motor.motor_value = (motor1 & nineBitMask) << 4;
- motor_buf[0] |= curr_motor.bytes[1];
- motor_buf[1] |= curr_motor.bytes[0];
-
- /* Set motor 2 */
- curr_motor.motor_value = (motor2 & nineBitMask) << 3;
- motor_buf[1] |= curr_motor.bytes[1];
- motor_buf[2] |= curr_motor.bytes[0];
-
- /* Set motor 3 */
- curr_motor.motor_value = (motor3 & nineBitMask) << 2;
- motor_buf[2] |= curr_motor.bytes[1];
- motor_buf[3] |= curr_motor.bytes[0];
-
- /* Set motor 4 */
- curr_motor.motor_value = (motor4 & nineBitMask) << 1;
- motor_buf[3] |= curr_motor.bytes[1];
- motor_buf[4] |= curr_motor.bytes[0];
-}
-
-void ar_enable_broadcast(int fd)
-{
- ar_select_motor(fd, 0);
-}
-
-int ar_multiplexing_init()
-{
- int fd;
-
- fd = open(GPIO_DEVICE_PATH, 0);
-
- if (fd < 0) {
- printf("GPIO: open fail\n");
- return fd;
- }
-
- /* deactivate all outputs */
- int ret = 0;
- ret += ioctl(fd, GPIO_SET, motor_gpios);
-
- if (ioctl(fd, GPIO_SET_OUTPUT, motor_gpios) != 0) {
- printf("GPIO: output set fail\n");
- close(fd);
- return -1;
- }
-
- if (ret < 0) {
- printf("GPIO: clearing pins fail\n");
- close(fd);
- return -1;
- }
-
- return fd;
-}
-
-int ar_multiplexing_deinit(int fd)
-{
- if (fd < 0) {
- printf("GPIO: no valid descriptor\n");
- return fd;
- }
-
- int ret = 0;
-
- /* deselect motor 1-4 */
- ret += ioctl(fd, GPIO_SET, motor_gpios);
-
- if (ret != 0) {
- printf("GPIO: clear failed %d times\n", ret);
- }
-
- if (ioctl(fd, GPIO_SET_INPUT, motor_gpios) != 0) {
- printf("GPIO: input set fail\n");
- return -1;
- }
-
- close(fd);
-
- return ret;
-}
-
-int ar_select_motor(int fd, uint8_t motor)
-{
- int ret = 0;
- unsigned long gpioset;
- /*
- * Four GPIOS:
- * GPIO_EXT1
- * GPIO_EXT2
- * GPIO_UART2_CTS
- * GPIO_UART2_RTS
- */
-
- /* select motor 0 to enable broadcast */
- if (motor == 0) {
- /* select motor 1-4 */
- ret += ioctl(fd, GPIO_CLEAR, motor_gpios);
-
- } else {
- /* deselect all */
- ret += ioctl(fd, GPIO_SET, motor_gpios);
-
- /* select reqested motor */
- ret += ioctl(fd, GPIO_CLEAR, motor_gpio[motor - 1]);
-
- /* deselect all others */
- // gpioset = motor_gpios ^ motor_gpio[motor - 1];
- // ret += ioctl(fd, GPIO_SET, gpioset);
- }
-
- return ret;
-}
-
-void ar_init_motors(int ardrone_uart, int *gpios_pin)
-{
- /* Initialize multiplexing */
- *gpios_pin = ar_multiplexing_init();
-
- /* Write ARDrone commands on UART2 */
- uint8_t initbuf[] = {0xE0, 0x91, 0xA1, 0x00, 0x40};
- uint8_t multicastbuf[] = {0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA0};
-
- /* initialize all motors
- * - select one motor at a time
- * - configure motor
- */
- int i;
- int errcounter = 0;
-
- for (i = 1; i < 5; ++i) {
- /* Initialize motors 1-4 */
- initbuf[3] = i;
- errcounter += ar_select_motor(*gpios_pin, i);
-
- write(ardrone_uart, initbuf + 0, 1);
-
- /* sleep 400 ms */
- usleep(200000);
- usleep(200000);
-
- write(ardrone_uart, initbuf + 1, 1);
- /* wait 50 ms */
- usleep(50000);
-
- write(ardrone_uart, initbuf + 2, 1);
- /* wait 50 ms */
- usleep(50000);
-
- write(ardrone_uart, initbuf + 3, 1);
- /* wait 50 ms */
- usleep(50000);
-
- write(ardrone_uart, initbuf + 4, 1);
- /* wait 50 ms */
- usleep(50000);
-
- /* enable multicast */
- write(ardrone_uart, multicastbuf + 0, 1);
- /* wait 1 ms */
- usleep(1000);
-
- write(ardrone_uart, multicastbuf + 1, 1);
- /* wait 1 ms */
- usleep(1000);
-
- write(ardrone_uart, multicastbuf + 2, 1);
- /* wait 1 ms */
- usleep(1000);
-
- write(ardrone_uart, multicastbuf + 3, 1);
- /* wait 1 ms */
- usleep(1000);
-
- write(ardrone_uart, multicastbuf + 4, 1);
- /* wait 1 ms */
- usleep(1000);
-
- write(ardrone_uart, multicastbuf + 5, 1);
- /* wait 5 ms */
- usleep(50000);
- }
-
- /* start the multicast part */
- errcounter += ar_select_motor(*gpios_pin, 0);
-
- if (errcounter != 0) {
- printf("AR: init sequence incomplete, failed %d times", -errcounter);
- fflush(stdout);
- }
-}
-
-/*
- * Sets the leds on the motor controllers, 1 turns led on, 0 off.
- */
-void ar_set_leds(int ardrone_uart, uint8_t led1_red, uint8_t led1_green, uint8_t led2_red, uint8_t led2_green, uint8_t led3_red, uint8_t led3_green, uint8_t led4_red, uint8_t led4_green)
-{
- /*
- * 2 bytes are sent. The first 3 bits describe the command: 011 means led control
- * the following 4 bits are the red leds for motor 4, 3, 2, 1
- * then 4 bits with unknown function, then 4 bits for green leds for motor 4, 3, 2, 1
- * the last bit is unknown.
- *
- * The packet is therefore:
- * 011 rrrr 0000 gggg 0
- */
- uint8_t leds[2];
- leds[0] = 0x60 | ((led4_red & 0x01) << 4) | ((led3_red & 0x01) << 3) | ((led2_red & 0x01) << 2) | ((led1_red & 0x01) << 1);
- leds[1] = ((led4_green & 0x01) << 4) | ((led3_green & 0x01) << 3) | ((led2_green & 0x01) << 2) | ((led1_green & 0x01) << 1);
- write(ardrone_uart, leds, 2);
-}
diff --git a/apps/multirotor_position_control/ardrone_motor_control.h b/apps/multirotor_position_control/ardrone_motor_control.h
deleted file mode 100644
index bf41e3f3b..000000000
--- a/apps/multirotor_position_control/ardrone_motor_control.h
+++ /dev/null
@@ -1,61 +0,0 @@
-/****************************************************************************
- * px4/ardrone_offboard_control.h
- *
- * Copyright (C) 2012 PX4 Autopilot Project. All rights reserved.
- * Author: Lorenz Meier <lm@inf.ethz.ch>
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- * notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- * notice, this list of conditions and the following disclaimer in
- * the documentation and/or other materials provided with the
- * distribution.
- * 3. Neither the name NuttX nor the names of its contributors may be
- * used to endorse or promote products derived from this software
- * without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
- * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
- * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
- * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
- * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
- * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
- ****************************************************************************/
-#include <nuttx/config.h>
-#include <pthread.h>
-#include <poll.h>
-#include <stdio.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <drivers/drv_gpio.h>
-
-
-/**
- * @brief Generate the 8-byte motor set packet
- *
- * @return the number of bytes (8)
- */
-void ar_get_motor_packet(uint8_t *motor_buf, uint16_t motor1, uint16_t motor2, uint16_t motor3, uint16_t motor4);
-
-int ar_select_motor(int fd, uint8_t motor);
-
-void ar_enable_broadcast(int fd);
-
-int ar_multiplexing_init(void);
-int ar_multiplexing_deinit(int fd);
-
-void ar_init_motors(int ardrone_uart, int *gpios_uart);
-
-void ar_set_leds(int ardrone_uart, uint8_t led1_red, uint8_t led1_green, uint8_t led2_red, uint8_t led2_green, uint8_t led3_red, uint8_t led3_green, uint8_t led4_red, uint8_t led4_green);
-
diff --git a/apps/multirotor_position_control/attitude_control.c b/apps/multirotor_position_control/attitude_control.c
deleted file mode 100644
index 8bcd33ede..000000000
--- a/apps/multirotor_position_control/attitude_control.c
+++ /dev/null
@@ -1,459 +0,0 @@
-/****************************************************************************
- *
- * Copyright (C) 2008-2012 PX4 Development Team. All rights reserved.
- * Author: Thomas Gubler <thomasgubler@student.ethz.ch>
- * Julian Oes <joes@student.ethz.ch>
- * Laurens Mackay <mackayl@student.ethz.ch>
- * Tobias Naegeli <naegelit@student.ethz.ch>
- * Martin Rutschmann <rutmarti@student.ethz.ch>
- * Lorenz Meier <lm@inf.ethz.ch>
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- * notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- * notice, this list of conditions and the following disclaimer in
- * the documentation and/or other materials provided with the
- * distribution.
- * 3. Neither the name PX4 nor the names of its contributors may be
- * used to endorse or promote products derived from this software
- * without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
- * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
- * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
- * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
- * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
- * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
- ****************************************************************************/
-
-/*
- * @file Implementation of attitude controller
- */
-
-#include "attitude_control.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <stdint.h>
-#include <stdbool.h>
-#include <px4/attitude_estimator_bm/matrix.h> //TODO: move matrix.h to somewhere else?
-#include "ardrone_motor_control.h"
-#include <float.h>
-#include <math.h>
-#include "pid.h"
-#include <arch/board/up_hrt.h>
-
-extern int ardrone_write;
-extern int gpios;
-
-#define CONTROL_PID_ATTITUDE_INTERVAL 5e-3f
-
-void turn_xy_plane(const float_vect3 *vector, float yaw,
- float_vect3 *result);
-void navi2body_xy_plane(const float_vect3 *vector, const float yaw,
- float_vect3 *result);
-
-void turn_xy_plane(const float_vect3 *vector, float yaw,
- float_vect3 *result)
-{
- //turn clockwise
- static uint16_t counter;
-
- result->x = (cosf(yaw) * vector->x + sinf(yaw) * vector->y);
- result->y = (-sinf(yaw) * vector->x + cosf(yaw) * vector->y);
- result->z = vector->z; //leave direction normal to xy-plane untouched
-
- counter++;
-}
-
-void navi2body_xy_plane(const float_vect3 *vector, const float yaw,
- float_vect3 *result)
-{
- turn_xy_plane(vector, yaw, result);
-// result->x = vector->x;
-// result->y = vector->y;
-// result->z = vector->z;
- // result->x = cos(yaw) * vector->x + sin(yaw) * vector->y;
- // result->y = -sin(yaw) * vector->x + cos(yaw) * vector->y;
- // result->z = vector->z; //leave direction normal to xy-plane untouched
-}
-
-void control_attitude(float roll, float pitch, float yaw, float thrust, const struct vehicle_attitude_s *att, const struct vehicle_status_s *status, int ardrone_pub, struct ardrone_control_s *ar_control)
-{
- static int motor_skip_counter = 0;
-
- static PID_t yaw_pos_controller;
- static PID_t yaw_speed_controller;
- static PID_t nick_controller;
- static PID_t roll_controller;
-
- const float min_thrust = 0.02f; /**< 2% minimum thrust */
- const float max_thrust = 1.0f; /**< 100% max thrust */
- const float scaling = 512.0f; /**< 100% thrust equals a value of 512 */
-
- const float min_gas = min_thrust * scaling;
- const float max_gas = max_thrust * scaling;
-
- static uint16_t motor_pwm[4] = {0, 0, 0, 0};
- static float motor_calc[4] = {0.0f, 0.0f, 0.0f, 0.0f};
-// static float remote_control_weight_z = 1;
-// static float position_control_weight_z = 0;
-
- static float pid_yawpos_lim;
- static float pid_yawspeed_lim;
- static float pid_att_lim;
-
- static bool initialized;
-
- static float_vect3 attitude_setpoint_navigationframe_from_positioncontroller;
- static commander_state_machine_t current_state;
-
- /* initialize the pid controllers when the function is called for the first time */
- if (initialized == false) {
-
- pid_init(&yaw_pos_controller,
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWPOS_P],
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWPOS_I],
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWPOS_D],
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWPOS_AWU],
- PID_MODE_DERIVATIV_CALC, 154);
-
- pid_init(&yaw_speed_controller,
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWSPEED_P],
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWSPEED_I],
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWSPEED_D],
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWSPEED_AWU],
- PID_MODE_DERIVATIV_CALC, 155);
-
- pid_init(&nick_controller,
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_P],
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_I],
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_D],
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_AWU],
- PID_MODE_DERIVATIV_SET, 156);
-
- pid_init(&roll_controller,
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_P],
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_I],
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_D],
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_AWU],
- PID_MODE_DERIVATIV_SET, 157);
-
- pid_yawpos_lim = global_data_parameter_storage->pm.param_values[PARAM_PID_YAWPOS_LIM];
- pid_yawspeed_lim = (max_gas - min_gas) * global_data_parameter_storage->pm.param_values[PARAM_PID_YAWSPEED_LIM];
- pid_att_lim = (max_gas - min_gas) * global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_LIM];
-
- //TODO: true initialization? get gps while on ground?
- attitude_setpoint_navigationframe_from_positioncontroller.x = 0.0f;
- attitude_setpoint_navigationframe_from_positioncontroller.y = 0.0f;
- attitude_setpoint_navigationframe_from_positioncontroller.z = 0.0f;
-
- initialized = true;
- }
-
- /* load new parameters with lower rate */
- if (motor_skip_counter % 50 == 0) {
- pid_set_parameters(&yaw_pos_controller,
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWPOS_P],
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWPOS_I],
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWPOS_D],
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWPOS_AWU]);
-
- pid_set_parameters(&yaw_speed_controller,
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWSPEED_P],
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWSPEED_I],
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWSPEED_D],
- global_data_parameter_storage->pm.param_values[PARAM_PID_YAWSPEED_AWU]);
-
- pid_set_parameters(&nick_controller,
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_P],
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_I],
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_D],
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_AWU]);
-
- pid_set_parameters(&roll_controller,
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_P],
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_I],
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_D],
- global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_AWU]);
-
- pid_yawpos_lim = global_data_parameter_storage->pm.param_values[PARAM_PID_YAWPOS_LIM];
- pid_yawspeed_lim = (max_gas - min_gas) * global_data_parameter_storage->pm.param_values[PARAM_PID_YAWSPEED_LIM];
- pid_att_lim = (max_gas - min_gas) * global_data_parameter_storage->pm.param_values[PARAM_PID_ATT_LIM];
- }
-
- current_state = status->state_machine;
- float_vect3 attitude_setpoint_bodyframe = {.x = 0.0f, .y = 0.0f, .z = 0.0f}; //this is the setpoint in the bodyframe "mixed" together from the setpoint from the remote and the setpoint from the position controller
-
- if (current_state == SYSTEM_STATE_AUTO) {
-
- attitude_setpoint_navigationframe_from_positioncontroller.x = ar_control->attitude_setpoint_navigationframe_from_positioncontroller[0];
- attitude_setpoint_navigationframe_from_positioncontroller.y = ar_control->attitude_setpoint_navigationframe_from_positioncontroller[1];
- attitude_setpoint_navigationframe_from_positioncontroller.z = ar_control->attitude_setpoint_navigationframe_from_positioncontroller[2];
-
- float yaw_e = att->yaw - attitude_setpoint_navigationframe_from_positioncontroller.z;
-
- // don't turn around the wrong side (only works if yaw angle is between +- 180 degree)
- if (yaw_e > M_PI_F) {
- yaw_e -= 2.0f * M_PI_F;
- }
-
- if (yaw_e < -M_PI_F) {
- yaw_e += 2.0f * M_PI_F;
- }
-
- attitude_setpoint_navigationframe_from_positioncontroller.z = pid_calculate(&yaw_pos_controller, 0.0f, yaw_e, 0.0f, CONTROL_PID_ATTITUDE_INTERVAL);
-
-
- /* limit control output */
- if (attitude_setpoint_navigationframe_from_positioncontroller.z > pid_yawpos_lim) {
- attitude_setpoint_navigationframe_from_positioncontroller.z = pid_yawpos_lim;
- yaw_pos_controller.saturated = 1;
- }
-
- if (attitude_setpoint_navigationframe_from_positioncontroller.z < -pid_yawpos_lim) {
- attitude_setpoint_navigationframe_from_positioncontroller.z = -pid_yawpos_lim;
- yaw_pos_controller.saturated = 1;
- }
-
- //transform attitude setpoint from position controller from navi to body frame on xy_plane
- float_vect3 attitude_setpoint_bodyframe_from_positioncontroller;
- navi2body_xy_plane(&attitude_setpoint_navigationframe_from_positioncontroller, att->yaw , &attitude_setpoint_bodyframe_from_positioncontroller); //yaw angle= att->yaw
- //now everything is in body frame
-
-
- //TODO: here we decide which input (position controller or ppm) we use. For now we have only the ppm, this should be decided dpending on the state machione (manula or auto) ppm should always overwrite auto (?)
- attitude_setpoint_bodyframe.x = attitude_setpoint_bodyframe_from_positioncontroller.x;
- attitude_setpoint_bodyframe.y = attitude_setpoint_bodyframe_from_positioncontroller.y;
- attitude_setpoint_bodyframe.z = attitude_setpoint_bodyframe_from_positioncontroller.z;
-
- } else if (current_state == SYSTEM_STATE_MANUAL) {
- attitude_setpoint_bodyframe.x = -roll * M_PI_F / 8.0f;
- attitude_setpoint_bodyframe.y = -pitch * M_PI_F / 8.0f;
- attitude_setpoint_bodyframe.z = -yaw * M_PI_F;
- }
-
- /* add an attitude offset which needs to be estimated somewhere */
- attitude_setpoint_bodyframe.x += global_data_parameter_storage->pm.param_values[PARAM_ATT_XOFFSET];
- attitude_setpoint_bodyframe.y += global_data_parameter_storage->pm.param_values[PARAM_ATT_YOFFSET];
-
- /*Calculate Controllers*/
- //control Nick
- float nick_control = pid_calculate(&nick_controller, attitude_setpoint_bodyframe.y, att->pitch, att->pitchspeed, CONTROL_PID_ATTITUDE_INTERVAL);
- //control Roll
- float roll_control = pid_calculate(&roll_controller, attitude_setpoint_bodyframe.x, att->roll, att->rollspeed, CONTROL_PID_ATTITUDE_INTERVAL);
- //control Yaw Speed
- float yaw_rate_control = pid_calculate(&yaw_speed_controller, attitude_setpoint_bodyframe.z, att->yawspeed, 0.0f, CONTROL_PID_ATTITUDE_INTERVAL); //attitude_setpoint_bodyframe.z is yaw speed!
-
- /*
- * compensate the vertical loss of thrust
- * when thrust plane has an angle.
- * start with a factor of 1.0 (no change)
- */
- float zcompensation = 1.0f;
-
- if (fabsf(att->roll) > 1.0f) {
- zcompensation *= 1.85081571768f;
-
- } else {
- zcompensation *= 1.0f / cosf(att->roll);
- }
-
- if (fabsf(att->pitch) > 1.0f) {
- zcompensation *= 1.85081571768f;
-
- } else {
- zcompensation *= 1.0f / cosf(att->pitch);
- }
-
- // use global_data.position_control_output.z and mix parameter global_data.param[PARAM_MIX_POSITION_Z_WEIGHT]
- // to compute thrust for Z position control
- //
- // float motor_thrust = min_gas +
- // ( ( 1 - global_data.param[PARAM_MIX_POSITION_Z_WEIGHT] ) * ( max_gas - min_gas ) * global_data.gas_remote * zcompensation )
- // + ( global_data.param[PARAM_MIX_POSITION_Z_WEIGHT] * ( max_gas - min_gas ) * controlled_thrust * zcompensation );
- //calculate the basic thrust
-
-
-
- float motor_thrust = 0;
-
- // FLYING MODES
- if (current_state == SYSTEM_STATE_MANUAL) {
- motor_thrust = thrust;
-
- } else if (current_state == SYSTEM_STATE_GROUND_READY ||
- current_state == SYSTEM_STATE_STABILIZED ||
- current_state == SYSTEM_STATE_AUTO ||
- current_state == SYSTEM_STATE_MISSION_ABORT) {
- motor_thrust = thrust; //TODO
-
- } else if (current_state == SYSTEM_STATE_EMCY_LANDING) {
- motor_thrust = thrust; //TODO
-
- } else if (current_state == SYSTEM_STATE_EMCY_CUTOFF) {
- /* immediately cut off motors */
- motor_thrust = 0;
-
- } else {
- /* limit motor throttle to zero for an unknown mode */
- motor_thrust = 0;
- }
-
- printf("t1:%1.3f ");
-
- /* compensate thrust vector for roll / pitch contributions */
- motor_thrust *= zcompensation;
-
- printf("t2:%1.3f\n");
-
- /* limit yaw rate output */
- if (yaw_rate_control > pid_yawspeed_lim) {
- yaw_rate_control = pid_yawspeed_lim;
- yaw_speed_controller.saturated = 1;
- }
-
- if (yaw_rate_control < -pid_yawspeed_lim) {
- yaw_rate_control = -pid_yawspeed_lim;
- yaw_speed_controller.saturated = 1;
- }
-
- if (nick_control > pid_att_lim) {
- nick_control = pid_att_lim;
- nick_controller.saturated = 1;
- }
-
- if (nick_control < -pid_att_lim) {
- nick_control = -pid_att_lim;
- nick_controller.saturated = 1;
- }
-
-
- if (roll_control > pid_att_lim) {
- roll_control = pid_att_lim;
- roll_controller.saturated = 1;
- }
-
- if (roll_control < -pid_att_lim) {
- roll_control = -pid_att_lim;
- roll_controller.saturated = 1;
- }
-
- /* Emit controller values */
- ar_control->setpoint_thrust_cast = motor_thrust;
- ar_control->setpoint_attitude[0] = attitude_setpoint_bodyframe.x;
- ar_control->setpoint_attitude[1] = attitude_setpoint_bodyframe.y;
- ar_control->setpoint_attitude[2] = attitude_setpoint_bodyframe.z;
- ar_control->attitude_control_output[0] = roll_control;
- ar_control->attitude_control_output[1] = nick_control;
- ar_control->attitude_control_output[2] = yaw_rate_control;
- ar_control->zcompensation = zcompensation;
- orb_publish(ORB_ID(ardrone_control), ardrone_pub, ar_control);
-
- float output_band = 0.f;
- float band_factor = 0.75f;
- const float startpoint_full_control = 0.25f; /**< start full control at 25% thrust */
- float yaw_factor = 1.0f;
-
- if (motor_thrust <= min_thrust) {
- motor_thrust = min_thrust;
- output_band = 0.f;
-
- } else if (motor_thrust < startpoint_full_control && motor_thrust > min_thrust) {
- output_band = band_factor * (motor_thrust - min_thrust);
-
- } else if (motor_thrust >= startpoint_full_control && motor_thrust < max_thrust - band_factor * startpoint_full_control) {
- output_band = band_factor * startpoint_full_control;
-
- } else if (motor_thrust >= max_thrust - band_factor * startpoint_full_control) {
- output_band = band_factor * (max_thrust - motor_thrust);
- }
-
- //add the yaw, nick and roll components to the basic thrust //TODO:this should be done by the mixer
-
- // FRONT (MOTOR 1)
- motor_calc[0] = motor_thrust + (roll_control / 2 + nick_control / 2 - yaw_rate_control);
-
- // RIGHT (MOTOR 2)
- motor_calc[1] = motor_thrust + (-roll_control / 2 + nick_control / 2 + yaw_rate_control);
-
- // BACK (MOTOR 3)
- motor_calc[2] = motor_thrust + (-roll_control / 2 - nick_control / 2 - yaw_rate_control);
-
- // LEFT (MOTOR 4)
- motor_calc[3] = motor_thrust + (roll_control / 2 - nick_control / 2 + yaw_rate_control);
-
- // if we are not in the output band
- if (!(motor_calc[0] < motor_thrust + output_band && motor_calc[0] > motor_thrust - output_band
- && motor_calc[1] < motor_thrust + output_band && motor_calc[1] > motor_thrust - output_band
- && motor_calc[2] < motor_thrust + output_band && motor_calc[2] > motor_thrust - output_band
- && motor_calc[3] < motor_thrust + output_band && motor_calc[3] > motor_thrust - output_band)) {
-
- yaw_factor = 0.5f;
- // FRONT (MOTOR 1)
- motor_calc[0] = motor_thrust + (roll_control / 2 + nick_control / 2 - yaw_rate_control * yaw_factor);
-
- // RIGHT (MOTOR 2)
- motor_calc[1] = motor_thrust + (-roll_control / 2 + nick_control / 2 + yaw_rate_control * yaw_factor);
-
- // BACK (MOTOR 3)
- motor_calc[2] = motor_thrust + (-roll_control / 2 - nick_control / 2 - yaw_rate_control * yaw_factor);
-
- // LEFT (MOTOR 4)
- motor_calc[3] = motor_thrust + (roll_control / 2 - nick_control / 2 + yaw_rate_control * yaw_factor);
- }
-
- for (int i = 0; i < 4; i++) {
- //check for limits
- if (motor_calc[i] < motor_thrust - output_band) {
- motor_calc[i] = motor_thrust - output_band;
- }
-
- if (motor_calc[i] > motor_thrust + output_band) {
- motor_calc[i] = motor_thrust + output_band;
- }
- }
-
- /* set the motor values */
-
- /* scale up from 0..1 to 10..512) */
- motor_pwm[0] = (uint16_t) motor_calc[0] * ((float)max_gas - min_gas) + min_gas;
- motor_pwm[1] = (uint16_t) motor_calc[1] * ((float)max_gas - min_gas) + min_gas;
- motor_pwm[2] = (uint16_t) motor_calc[2] * ((float)max_gas - min_gas) + min_gas;
- motor_pwm[3] = (uint16_t) motor_calc[3] * ((float)max_gas - min_gas) + min_gas;
-
- /* Keep motors spinning while armed and prevent overflows */
-
- /* Failsafe logic - should never be necessary */
- motor_pwm[0] = (motor_pwm[0] > 0) ? motor_pwm[0] : 10;
- motor_pwm[1] = (motor_pwm[1] > 0) ? motor_pwm[1] : 10;
- motor_pwm[2] = (motor_pwm[2] > 0) ? motor_pwm[2] : 10;
- motor_pwm[3] = (motor_pwm[3] > 0) ? motor_pwm[3] : 10;
-
- /* Failsafe logic - should never be necessary */
- motor_pwm[0] = (motor_pwm[0] <= 512) ? motor_pwm[0] : 512;
- motor_pwm[1] = (motor_pwm[1] <= 512) ? motor_pwm[1] : 512;
- motor_pwm[2] = (motor_pwm[2] <= 512) ? motor_pwm[2] : 512;
- motor_pwm[3] = (motor_pwm[3] <= 512) ? motor_pwm[3] : 512;
-
- /* send motors via UART */
- if (motor_skip_counter % 5 == 0) {
- uint8_t motorSpeedBuf[5] = {1, 2, 3, 4, 5};
- ar_get_motor_packet(motorSpeedBuf, motor_pwm[0], motor_pwm[1], motor_pwm[2], motor_pwm[3]);
- write(ardrone_write, motorSpeedBuf, 5);
- }
-
- motor_skip_counter++;
-
-}
diff --git a/apps/multirotor_position_control/attitude_control.h b/apps/multirotor_position_control/attitude_control.h
deleted file mode 100644
index a93a511ba..000000000
--- a/apps/multirotor_position_control/attitude_control.h
+++ /dev/null
@@ -1,52 +0,0 @@
-/****************************************************************************
- *
- * Copyright (C) 2012 PX4 Development Team. All rights reserved.
- * Author: Thomas Gubler <thomasgubler@student.ethz.ch>
- * Julian Oes <joes@student.ethz.ch>
- *
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- * notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- * notice, this list of conditions and the following disclaimer in
- * the documentation and/or other materials provided with the
- * distribution.
- * 3. Neither the name PX4 nor the names of its contributors may be
- * used to endorse or promote products derived from this software
- * without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
- * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
- * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
- * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
- * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
- * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
- ****************************************************************************/
-
-/* @file attitude control for quadrotors */
-
-#ifndef ATTITUDE_CONTROL_H_
-#define ATTITUDE_CONTROL_H_
-
-#include <uORB/uORB.h>
-#include <uORB/topics/rc_channels.h>
-#include <uORB/topics/vehicle_attitude.h>
-#include <uORB/topics/ardrone_control.h>
-#include <uORB/topics/vehicle_status.h>
-
-void control_attitude(float roll, float pitch, float yaw, float thrust, const struct vehicle_attitude_s *att,
- const struct vehicle_status_s *status, int ardrone_pub,
- struct ardrone_control_s *ar_control);
-
-#endif /* ATTITUDE_CONTROL_H_ */
diff --git a/apps/multirotor_position_control/multirotor_position_control.c b/apps/multirotor_position_control/multirotor_position_control.c
new file mode 100644
index 000000000..52930fd83
--- /dev/null
+++ b/apps/multirotor_position_control/multirotor_position_control.c
@@ -0,0 +1,136 @@
+/****************************************************************************
+ *
+ * Copyright (C) 2008-2012 PX4 Development Team. All rights reserved.
+ * Author: Lorenz Meier <lm@inf.ethz.ch>
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ * notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in
+ * the documentation and/or other materials provided with the
+ * distribution.
+ * 3. Neither the name PX4 nor the names of its contributors may be
+ * used to endorse or promote products derived from this software
+ * without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
+ * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
+ * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+ * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ ****************************************************************************/
+
+/*
+ * @file Implementation of AR.Drone 1.0 / 2.0 control interface
+ */
+
+#include <nuttx/config.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <stdbool.h>
+#include <unistd.h>
+#include <fcntl.h>
+#include <errno.h>
+#include <debug.h>
+#include <termios.h>
+#include <time.h>
+#include <sys/prctl.h>
+#include <arch/board/up_hrt.h>
+#include "ardrone_control.h"
+#include "attitude_control.h"
+#include "rate_control.h"
+#include "ardrone_motor_control.h"
+#include "position_control.h"
+#include <uORB/uORB.h>
+#include <uORB/topics/vehicle_status.h>
+#include <uORB/topics/vehicle_attitude.h>
+#include <uORB/topics/manual_control_setpoint.h>
+
+__EXPORT int multirotor_position_control_main(int argc, char *argv[]);
+
+static bool thread_should_exit;
+static bool thread_running = false;
+static int mpc_task;
+
+static int
+mpc_thread_main(int argc, char *argv[])
+{
+ /* welcome user */
+ printf("[multirotor pos ctrl] Control started, taking over position control\n");
+
+ /* structures */
+ struct vehicle_status_s state;
+ struct vehicle_attitude_s att;
+ //struct vehicle_global_position_setpoint_s global_pos_sp;
+ struct vehicle_local_position_setpoint_s local_pos_sp;
+ struct vehicle_local_position_s local_pos;
+ struct manual_control_setpoint_s manual;
+ struct vehicle_attitude_setpoint_s att_sp;
+
+ /* subscribe to attitude, motor setpoints and system state */
+ int att_sub = orb_subscribe(ORB_ID(vehicle_attitude));
+ int state_sub = orb_subscribe(ORB_ID(vehicle_status));
+ int manual_sub = orb_subscribe(ORB_ID(manual_control_setpoint));
+ int local_pos_sub = orb_subscribe(ORB_ID(vehicle_local_position));
+ //int global_pos_sp_sub = orb_subscribe(ORB_ID(vehicle_global_position_setpoint));
+ int local_pos_sp_sub = orb_subscribe(ORB_ID(vehicle_local_position_setpoint));
+
+ /* publish attitude setpoint */
+ int att_sp_pub = orb_advertise(ORB_ID(vehicle_attitude_setpoint), &att_sp);
+
+ while (1) {
+ /* get a local copy of the vehicle state */
+ orb_copy(ORB_ID(vehicle_status), state_sub, &state);
+ /* get a local copy of manual setpoint */
+ orb_copy(ORB_ID(manual_control_setpoint), manual_sub, &manual);
+ /* get a local copy of attitude */
+ orb_copy(ORB_ID(vehicle_attitude), att_sub, &att);
+ /* get a local copy of local position */
+ orb_copy(ORB_ID(vehicle_local_position), local_pos_sub, &local_pos);
+ /* get a local copy of local position setpoint */
+ orb_copy(ORB_ID(vehicle_local_position_setpoint), local_pos_sp_sub, &local_pos_sp);
+
+ if (state.state_machine == SYSTEM_STATE_AUTO) {
+ position_control(&state, &manual, &att, &local_pos, &local_pos_sp, &att_sp);
+ /* publish new attitude setpoint */
+ orb_publish(ORB_ID(vehicle_attitude_setpoint), att_sp_pub, &att_sp);
+ } else if (state.state_machine == SYSTEM_STATE_STABILIZE) {
+ /* set setpoint to current position */
+ // XXX select pos reset channel on remote
+ /* reset setpoint to current position (position hold) */
+ // if (1 == 2) {
+ // local_pos_sp.x = local_pos.x;
+ // local_pos_sp.y = local_pos.y;
+ // local_pos_sp.z = local_pos.z;
+ // local_pos_sp.yaw = att.yaw;
+ // }
+ }
+
+ /* run at approximately 50 Hz */
+ usleep(20000);
+
+ counter++;
+ }
+
+ /* close uarts */
+ close(ardrone_write);
+ ar_multiplexing_deinit(gpios);
+
+ printf("[ardrone_control] ending now...\r\n");
+ fflush(stdout);
+ return 0;
+}
+
diff --git a/apps/multirotor_position_control/pid.c b/apps/multirotor_position_control/pid.c
deleted file mode 100644
index 5ce05670e..000000000
--- a/apps/multirotor_position_control/pid.c
+++ /dev/null
@@ -1,109 +0,0 @@
-#include "pid.h"
-
-#include <px4/attitude_estimator_bm/matrix.h> //TODO: move matrix.h to somewhere else?
-
-void pid_init(PID_t *pid, float kp, float ki, float kd, float intmax,
- uint8_t mode, uint8_t plot_i)
-{
- pid->kp = kp;
- pid->ki = ki;
- pid->kd = kd;
- pid->intmax = intmax;
- pid->mode = mode;
- pid->plot_i = plot_i;
- pid->count = 0;
- pid->saturated = 0;
-
- pid->sp = 0;
- pid->error_previous = 0;
- pid->integral = 0;
-}
-void pid_set_parameters(PID_t *pid, float kp, float ki, float kd, float intmax)
-{
- pid->kp = kp;
- pid->ki = ki;
- pid->kd = kd;
- pid->intmax = intmax;
- // pid->mode = mode;
-
- // pid->sp = 0;
- // pid->error_previous = 0;
- // pid->integral = 0;
-}
-
-//void pid_set(PID_t *pid, float sp)
-//{
-// pid->sp = sp;
-// pid->error_previous = 0;
-// pid->integral = 0;
-//}
-
-/**
- *
- * @param pid
- * @param val
- * @param dt
- * @return
- */
-float pid_calculate(PID_t *pid, float sp, float val, float val_dot, float dt)
-{
- /* error = setpoint - actual_position
- integral = integral + (error*dt)
- derivative = (error - previous_error)/dt
- output = (Kp*error) + (Ki*integral) + (Kd*derivative)
- previous_error = error
- wait(dt)
- goto start
- */
-
- float i, d;
- pid->sp = sp;
- float error = pid->sp - val;
-
- if (pid->saturated && (pid->integral * error > 0)) {
- //Output is saturated and the integral would get bigger (positive or negative)
- i = pid->integral;
-
- //Reset saturation. If we are still saturated this will be set again at output limit check.
- pid->saturated = 0;
-
- } else {
- i = pid->integral + (error * dt);
- }
-
- // Anti-Windup. Needed if we don't use the saturation above.
- if (pid->intmax != 0.0) {
- if (i > pid->intmax) {
- pid->integral = pid->intmax;
-
- } else if (i < -pid->intmax) {
-
- pid->integral = -pid->intmax;
-
- } else {
- pid->integral = i;
- }
-
- //Send Controller integrals
- // Disabled because of new possibilities with debug_vect.
- // Now sent in Main Loop at 5 Hz. 26.06.2010 Laurens
- // if (pid->plot_i && (pid->count++ % 16 == 0)&&(global_data.param[PARAM_SEND_SLOT_DEBUG_2] == 1))
- // {
- // mavlink_msg_debug_send(MAVLINK_COMM_1, pid->plot_i, pid->integral);
- // }
- }
-
- if (pid->mode == PID_MODE_DERIVATIV_CALC) {
- d = (error - pid->error_previous) / dt;
-
- } else if (pid->mode == PID_MODE_DERIVATIV_SET) {
- d = -val_dot;
-
- } else {
- d = 0;
- }
-
- pid->error_previous = error;
-
- return (error * pid->kp) + (i * pid->ki) + (d * pid->kd);
-}
diff --git a/apps/multirotor_position_control/pid.h b/apps/multirotor_position_control/pid.h
deleted file mode 100644
index a721c839c..000000000
--- a/apps/multirotor_position_control/pid.h
+++ /dev/null
@@ -1,40 +0,0 @@
-/*
- * pid.h
- *
- * Created on: May 29, 2012
- * Author: thomasgubler
- */
-
-#ifndef PID_H_
-#define PID_H_
-
-#include <stdint.h>
-
-/* PID_MODE_DERIVATIV_CALC calculates discrete derivative from previous error
- * val_dot in pid_calculate() will be ignored */
-#define PID_MODE_DERIVATIV_CALC 0
-/* Use PID_MODE_DERIVATIV_SET if you have the derivative already (Gyros, Kalman) */
-#define PID_MODE_DERIVATIV_SET 1
-
-typedef struct {
- float kp;
- float ki;
- float kd;
- float intmax;
- float sp;
- float integral;
- float error_previous;
- uint8_t mode;
- uint8_t plot_i;
- uint8_t count;
- uint8_t saturated;
-} PID_t;
-
-void pid_init(PID_t *pid, float kp, float ki, float kd, float intmax, uint8_t mode, uint8_t plot_i);
-void pid_set_parameters(PID_t *pid, float kp, float ki, float kd, float intmax);
-//void pid_set(PID_t *pid, float sp);
-float pid_calculate(PID_t *pid, float sp, float val, float val_dot, float dt);
-
-
-
-#endif /* PID_H_ */
diff --git a/apps/multirotor_position_control/position_control.c b/apps/multirotor_position_control/position_control.c
index 6ba165459..fd731a33d 100644
--- a/apps/multirotor_position_control/position_control.c
+++ b/apps/multirotor_position_control/position_control.c
@@ -1,8 +1,10 @@
/****************************************************************************
- * ardrone_control/position_control.c
*
- * Copyright (C) 2008, 2012 Thomas Gubler, Julian Oes, Lorenz Meier. All rights reserved.
- * Author: Based on the pixhawk quadrotor controller
+ * Copyright (C) 2008-2012 PX4 Development Team. All rights reserved.
+ * Author: @author Lorenz Meier <lm@inf.ethz.ch>
+ * @author Laurens Mackay <mackayl@student.ethz.ch>
+ * @author Tobias Naegeli <naegelit@student.ethz.ch>
+ * @author Martin Rutschmann <rutmarti@student.ethz.ch>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
@@ -14,7 +16,7 @@
* notice, this list of conditions and the following disclaimer in
* the documentation and/or other materials provided with the
* distribution.
- * 3. Neither the name NuttX nor the names of its contributors may be
+ * 3. Neither the name PX4 nor the names of its contributors may be
* used to endorse or promote products derived from this software
* without specific prior written permission.
*
@@ -33,7 +35,11 @@
*
****************************************************************************/
-#include "position_control.h"
+/**
+ * @file multirotor_position_control.c
+ * Implementation of the position control for a multirotor VTOL
+ */
+
#include <stdio.h>
#include <stdlib.h>
#include <stdio.h>
@@ -41,41 +47,11 @@
#include <math.h>
#include <stdbool.h>
#include <float.h>
-#include "pid.h"
-
-#ifndef FM_PI
-#define FM_PI 3.1415926535897932384626433832795f
-#endif
-
-
+#include <systemlib/pid/pid.h>
-#define CONTROL_PID_POSITION_INTERVAL 0.020
+#include "multirotor_position_control.h"
-int read_lock_position(global_data_position_t *position)
-{
- static int ret;
- ret = global_data_wait(&global_data_position->access_conf);
-
- if (ret == 0) {
- memcpy(&position->lat, &global_data_position->lat, sizeof(global_data_position->lat));
- memcpy(&position->lon, &global_data_position->lon, sizeof(global_data_position->lon));
- memcpy(&position->alt, &global_data_position->alt, sizeof(global_data_position->alt));
- memcpy(&position->relative_alt, &global_data_position->relative_alt, sizeof(global_data_position->relative_alt));
- memcpy(&position->vx, &global_data_position->vx, sizeof(global_data_position->vx));
- memcpy(&position->vy, &global_data_position->vy, sizeof(global_data_position->vy));
- memcpy(&position->vz, &global_data_position->vz, sizeof(global_data_position->vz));
- memcpy(&position->hdg, &global_data_position->hdg, sizeof(global_data_position->hdg));
-
-
- } else {
- printf("Controller timeout, no new position values available\n");
- }
-
- global_data_unlock(&global_data_position->access_conf);
- return ret;
-}
-
-float get_distance_to_next_waypoint(float lat_now, float lon_now, float lat_next, float lon_next)
+float get_distance_to_next_waypoint(double lat_now, double lon_now, double lat_next, double lon_next)
{
double lat_now_rad = lat_now / 180.0 * M_PI;
double lon_now_rad = lon_now / 180.0 * M_PI;
@@ -94,7 +70,7 @@ float get_distance_to_next_waypoint(float lat_now, float lon_now, float lat_next
return radius_earth * c;
}
-float get_bearing_to_next_waypoint(float lat_now, float lon_now, float lat_next, float lon_next)
+float get_bearing_to_next_waypoint(double lat_now, double lon_now, double lat_next, double lon_next)
{
double lat_now_rad = lat_now / 180.0 * M_PI;
double lon_now_rad = lon_now / 180.0 * M_PI;
@@ -104,17 +80,20 @@ float get_bearing_to_next_waypoint(float lat_now, float lon_now, float lat_next,
double d_lat = lat_next_rad - lat_now_rad;
double d_lon = lon_next_rad - lon_now_rad;
-
+ /* conscious mix of double and float trig function to maximize speed and efficiency */
float theta = atan2f(sin(d_lon) * cos(lat_next_rad) , cos(lat_now_rad) * sin(lat_next_rad) - sin(lat_now_rad) * cos(lat_next_rad) * cos(d_lon));
- if (theta < 0) {
- theta = theta + 2 * FM_PI;
+ // XXX wrapping check is incomplete
+ if (theta < 0.0f) {
+ theta = theta + 2.0f * M_PI_F;
}
return theta;
}
-void control_position(void)
+void control_multirotor_position(const struct vehicle_state_s *vstatus, const struct vehicle_manual_control_s *manual,
+ const struct vehicle_attitude_s *att, const struct vehicle_local_position_s *local_pos,
+ const struct vehicle_local_position_setpoint_s *local_pos_sp, struct vehicle_attitude_setpoint_s *att_sp)
{
static PID_t distance_controller;
@@ -136,8 +115,6 @@ void control_position(void)
if (initialized == false) {
- global_data_lock(&global_data_parameter_storage->access_conf);
-
pid_init(&distance_controller,
global_data_parameter_storage->pm.param_values[PARAM_PID_POS_P],
global_data_parameter_storage->pm.param_values[PARAM_PID_POS_I],
@@ -148,10 +125,6 @@ void control_position(void)
// pid_pos_lim = global_data_parameter_storage->pm.param_values[PARAM_PID_POS_LIM];
// pid_pos_z_lim = global_data_parameter_storage->pm.param_values[PARAM_PID_POS_Z_LIM];
- pm_counter = global_data_parameter_storage->counter;
-
- global_data_unlock(&global_data_parameter_storage->access_conf);
-
thrust_total = 0.0f;
/* Position initialization */
@@ -295,14 +268,8 @@ void control_position(void)
roll_desired = asinf(roll_fraction);
}
- /*Broadcast desired angles */
- global_data_lock(&global_data_ardrone_control->access_conf);
- global_data_ardrone_control->attitude_setpoint_navigationframe_from_positioncontroller[0] = roll_desired;
- global_data_ardrone_control->attitude_setpoint_navigationframe_from_positioncontroller[1] = pitch_desired;
- global_data_ardrone_control->attitude_setpoint_navigationframe_from_positioncontroller[2] = bearing; //TODO: add yaw setpoint
- global_data_unlock(&global_data_ardrone_control->access_conf);
- global_data_broadcast(&global_data_ardrone_control->access_conf);
-
+ att_sp.roll = roll_desired;
+ att_sp.pitch = pitch_desired;
counter++;
}
diff --git a/apps/multirotor_position_control/position_control.h b/apps/multirotor_position_control/position_control.h
index dbc0650e6..5ba59362e 100644
--- a/apps/multirotor_position_control/position_control.h
+++ b/apps/multirotor_position_control/position_control.h
@@ -1,13 +1,50 @@
-/*
- * position_control.h
+/****************************************************************************
*
- * Created on: May 29, 2012
- * Author: thomasgubler
+ * Copyright (C) 2008-2012 PX4 Development Team. All rights reserved.
+ * Author: @author Lorenz Meier <lm@inf.ethz.ch>
+ * @author Laurens Mackay <mackayl@student.ethz.ch>
+ * @author Tobias Naegeli <naegelit@student.ethz.ch>
+ * @author Martin Rutschmann <rutmarti@student.ethz.ch>
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ * notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in
+ * the documentation and/or other materials provided with the
+ * distribution.
+ * 3. Neither the name PX4 nor the names of its contributors may be
+ * used to endorse or promote products derived from this software
+ * without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
+ * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
+ * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+ * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ ****************************************************************************/
+
+/**
+ * @file multirotor_position_control.h
+ * Definition of the position control for a multirotor VTOL
*/
#ifndef POSITION_CONTROL_H_
#define POSITION_CONTROL_H_
-void control_position(void);
+void control_multirotor_position(const struct vehicle_state_s *vstatus, const struct vehicle_manual_control_s *manual,
+ const struct vehicle_attitude_s *att, const struct vehicle_local_position_s *local_pos,
+ const struct vehicle_local_position_setpoint_s *local_pos_sp, struct vehicle_attitude_setpoint_s *att_sp);
#endif /* POSITION_CONTROL_H_ */
diff --git a/apps/multirotor_position_control/rate_control.c b/apps/multirotor_position_control/rate_control.c
deleted file mode 100644
index 4abba6255..000000000
--- a/apps/multirotor_position_control/rate_control.c
+++ /dev/null
@@ -1,320 +0,0 @@
-/****************************************************************************
- *
- * Copyright (C) 2008-2012 PX4 Development Team. All rights reserved.
- * Author: Tobias Naegeli <nagelit@student.ethz.ch>
- * Lorenz Meier <lm@inf.ethz.ch>
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- * notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- * notice, this list of conditions and the following disclaimer in
- * the documentation and/or other materials provided with the
- * distribution.
- * 3. Neither the name PX4 nor the names of its contributors may be
- * used to endorse or promote products derived from this software
- * without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
- * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
- * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
- * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
- * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
- * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- *
- ****************************************************************************/
-
-/*
- * @file Implementation of attitude rate control
- */
-
-#include "rate_control.h"
-#include "ardrone_control_helper.h"
-#include "ardrone_motor_control.h"
-#include <arch/board/up_hrt.h>
-
-extern int ardrone_write;
-extern int gpios;
-
-typedef struct {
- uint16_t motor_front_nw; ///< Front motor in + configuration, front left motor in x configuration
- uint16_t motor_right_ne; ///< Right motor in + configuration, front right motor in x configuration
- uint16_t motor_back_se; ///< Back motor in + configuration, back right motor in x configuration
- uint16_t motor_left_sw; ///< Left motor in + configuration, back left motor in x configuration
- uint8_t target_system; ///< System ID of the system that should set these motor commands
-} quad_motors_setpoint_t;
-
-
-void control_rates(struct sensor_combined_s *raw, struct ardrone_motors_setpoint_s *setpoints)
-{
- static quad_motors_setpoint_t actuators_desired;
- //static quad_motors_setpoint_t quad_motors_setpoint_desired;
-
- static int16_t outputBand = 0;
-
-// static uint16_t control_counter;
- static hrt_abstime now_time;
- static hrt_abstime last_time;
-
- static float setpointXrate;
- static float setpointYrate;
- static float setpointZrate;
-
- static float setpointRateCast[3];
- static float Kp;
-// static float Ki;
- static float setpointThrustCast;
- static float startpointFullControll;
- static float maxThrustSetpoints;
-
- static float gyro_filtered[3];
- static float gyro_filtered_offset[3];
- static float gyro_alpha;
- static float gyro_alpha_offset;
-// static float errXrate;
- static float attRatesScaled[3];
-
- static uint16_t offsetCnt;
-// static float antiwindup;
- static int motor_skip_counter;
-
- static int read_ret;
-
- static bool initialized;
-
- if (initialized == false) {
- initialized = true;
-
- /* Read sensors for initial values */
-
- gyro_filtered_offset[0] = 0.00026631611f * (float)raw->gyro_raw[0];
- gyro_filtered_offset[1] = 0.00026631611f * (float)raw->gyro_raw[1];
- gyro_filtered_offset[2] = 0.00026631611f * (float)raw->gyro_raw[2];
-
- gyro_filtered[0] = 0.00026631611f * (float)raw->gyro_raw[0];
- gyro_filtered[1] = 0.00026631611f * (float)raw->gyro_raw[1];
- gyro_filtered[2] = 0.00026631611f * (float)raw->gyro_raw[2];
-
- outputBand = 0;
- startpointFullControll = 150.0f;
- maxThrustSetpoints = 511.0f;
- //Kp=60;
- //Kp=40.0f;
- //Kp=45;
- Kp = 30.0f;
-// Ki=0.0f;
-// antiwindup=50.0f;
- }
-
- /* Get setpoint */
-
-
- //Rate Controller
- setpointRateCast[0] = -((float)setpoints->motor_right_ne - 9999.0f) * 0.01f / 180.0f * 3.141f;
- setpointRateCast[1] = -((float)setpoints->motor_front_nw - 9999.0f) * 0.01f / 180.0f * 3.141f;
- setpointRateCast[2] = 0; //-((float)setpoints->motor_back_se-9999.0f)*0.01f;
- //Ki=actuatorDesired.motorRight_NE*0.001f;
- setpointThrustCast = setpoints->motor_left_sw;
-
- attRatesScaled[0] = 0.000317603994f * (float)raw->gyro_raw[0];
- attRatesScaled[1] = 0.000317603994f * (float)raw->gyro_raw[1];
- attRatesScaled[2] = 0.000317603994f * (float)raw->gyro_raw[2];
-
- //filtering of the gyroscope values
-
- //compute filter coefficient alpha
-
- //gyro_alpha=0.005/(2.0f*3.1415f*200.0f+0.005f);
- //gyro_alpha=0.009;
- gyro_alpha = 0.09f;
- gyro_alpha_offset = 0.001f;
- //gyro_alpha=0.001;
- //offset estimation and filtering
- offsetCnt++;
- uint8_t i;
-
- for (i = 0; i < 3; i++) {
- if (offsetCnt < 5000) {
- gyro_filtered_offset[i] = attRatesScaled[i] * gyro_alpha_offset + gyro_filtered_offset[i] * (1 - gyro_alpha_offset);
- }
-
- gyro_filtered[i] = 1.0f * ((attRatesScaled[i] - gyro_filtered_offset[i]) * gyro_alpha + gyro_filtered[i] * (1 - gyro_alpha)) - 0 * setpointRateCast[i];
- }
-
- // //START DEBUG
- // /* write filtered values to global_data_attitude */
- // global_data_attitude->rollspeed = gyro_filtered[0];
- // global_data_attitude->pitchspeed = gyro_filtered[1];
- // global_data_attitude->yawspeed = gyro_filtered[2];
- // //END DEBUG
-
- //rate controller
-
- //X-axis
- setpointXrate = -Kp * (setpointRateCast[0] - gyro_filtered[0]);
- //Y-axis
- setpointYrate = -Kp * (setpointRateCast[1] - gyro_filtered[1]);
- //Z-axis
- setpointZrate = -Kp * (setpointRateCast[2] - gyro_filtered[2]);
-
-
-
-
- //Mixing
- if (setpointThrustCast <= 0) {
- setpointThrustCast = 0;
- outputBand = 0;
- }
-
- if ((setpointThrustCast < startpointFullControll) && (setpointThrustCast > 0)) {
- outputBand = 0.75f * setpointThrustCast;
- }
-
- if ((setpointThrustCast >= startpointFullControll) && (setpointThrustCast < maxThrustSetpoints - 0.75f * startpointFullControll)) {
- outputBand = 0.75f * startpointFullControll;
- }
-
- if (setpointThrustCast >= maxThrustSetpoints - 0.75f * startpointFullControll) {
- setpointThrustCast = 0.75f * startpointFullControll;
- outputBand = 0.75f * startpointFullControll;
- }
-
- actuators_desired.motor_front_nw = setpointThrustCast + (setpointXrate + setpointYrate + setpointZrate);
- actuators_desired.motor_right_ne = setpointThrustCast + (-setpointXrate + setpointYrate - setpointZrate);
- actuators_desired.motor_back_se = setpointThrustCast + (-setpointXrate - setpointYrate + setpointZrate);
- actuators_desired.motor_left_sw = setpointThrustCast + (setpointXrate - setpointYrate - setpointZrate);
-
-
- if ((setpointThrustCast + setpointXrate + setpointYrate + setpointZrate) > (setpointThrustCast + outputBand)) {
- actuators_desired.motor_front_nw = setpointThrustCast + outputBand;
- }
-
- if ((setpointThrustCast + setpointXrate + setpointYrate + setpointZrate) < (setpointThrustCast - outputBand)) {
- actuators_desired.motor_front_nw = setpointThrustCast - outputBand;
- }
-
- if ((setpointThrustCast + (-setpointXrate) + setpointYrate - setpointZrate) > (setpointThrustCast + outputBand)) {
- actuators_desired.motor_right_ne = setpointThrustCast + outputBand;
- }
-
- if ((setpointThrustCast + (-setpointXrate) + setpointYrate - setpointZrate) < (setpointThrustCast - outputBand)) {
- actuators_desired.motor_right_ne = setpointThrustCast - outputBand;
- }
-
- if ((setpointThrustCast + (-setpointXrate) + (-setpointYrate) + setpointZrate) > (setpointThrustCast + outputBand)) {
- actuators_desired.motor_back_se = setpointThrustCast + outputBand;
- }
-
- if ((setpointThrustCast + (-setpointXrate) + (-setpointYrate) + setpointZrate) < (setpointThrustCast - outputBand)) {
- actuators_desired.motor_back_se = setpointThrustCast - outputBand;
- }
-
- if ((setpointThrustCast + setpointXrate + (-setpointYrate) - setpointZrate) > (setpointThrustCast + outputBand)) {
- actuators_desired.motor_left_sw = setpointThrustCast + outputBand;
- }
-
- if ((setpointThrustCast + setpointXrate + (-setpointYrate) - setpointZrate) < (setpointThrustCast - outputBand)) {
- actuators_desired.motor_left_sw = setpointThrustCast - outputBand;
- }
-
- //printf("%lu,%lu,%lu,%lu\n",actuators_desired.motor_front_nw, actuators_desired.motor_right_ne, actuators_desired.motor_back_se, actuators_desired.motor_left_sw);
-
- if (motor_skip_counter % 5 == 0) {
- uint8_t motorSpeedBuf[5];
- ar_get_motor_packet(motorSpeedBuf, actuators_desired.motor_front_nw, actuators_desired.motor_right_ne, actuators_desired.motor_back_se, actuators_desired.motor_left_sw);
-// uint8_t* motorSpeedBuf = ar_get_motor_packet(1, 1, 1, 1);
-// if(motor_skip_counter %50 == 0)
-// {
-// if(0==actuators_desired.motor_front_nw || 0 == actuators_desired.motor_right_ne || 0 == actuators_desired.motor_back_se || 0 == actuators_desired.motor_left_sw)
-// printf("Motors set: %u, %u, %u, %u\n", actuators_desired.motor_front_nw, actuators_desired.motor_right_ne, actuators_desired.motor_back_se, actuators_desired.motor_left_sw);
-// printf("input: %u\n", setpoints->motor_front_nw);
-// printf("Roll casted desired: %f, Pitch casted desired: %f, Yaw casted desired: %f\n", setpointRateCast[0], setpointRateCast[1], setpointRateCast[2]);
-// }
- write(ardrone_write, motorSpeedBuf, 5);
-// motor_skip_counter = 0;
- }
-
- motor_skip_counter++;
-
- //START DEBUG
-// global_data_lock(&global_data_ardrone_control->access_conf);
-// global_data_ardrone_control->timestamp = hrt_absolute_time();
-// global_data_ardrone_control->gyro_scaled[0] = attRatesScaled[0];
-// global_data_ardrone_control->gyro_scaled[1] = attRatesScaled[1];
-// global_data_ardrone_control->gyro_scaled[2] = attRatesScaled[2];
-// global_data_ardrone_control->gyro_filtered[0] = gyro_filtered[0];
-// global_data_ardrone_control->gyro_filtered[1] = gyro_filtered[1];
-// global_data_ardrone_control->gyro_filtered[2] = gyro_filtered[2];
-// global_data_ardrone_control->gyro_filtered_offset[0] = gyro_filtered_offset[0];
-// global_data_ardrone_control->gyro_filtered_offset[1] = gyro_filtered_offset[1];
-// global_data_ardrone_control->gyro_filtered_offset[2] = gyro_filtered_offset[2];
-// global_data_ardrone_control->setpoint_rate_cast[0] = setpointRateCast[0];
-// global_data_ardrone_control->setpoint_rate_cast[1] = setpointRateCast[1];
-// global_data_ardrone_control->setpoint_rate_cast[2] = setpointRateCast[2];
-// global_data_ardrone_control->setpoint_thrust_cast = setpointThrustCast;
-// global_data_ardrone_control->setpoint_rate[0] = setpointXrate;
-// global_data_ardrone_control->setpoint_rate[1] = setpointYrate;
-// global_data_ardrone_control->setpoint_rate[2] = setpointZrate;
-// global_data_ardrone_control->motor_front_nw = actuators_desired.motor_front_nw;
-// global_data_ardrone_control->motor_right_ne = actuators_desired.motor_right_ne;
-// global_data_ardrone_control->motor_back_se = actuators_desired.motor_back_se;
-// global_data_ardrone_control->motor_left_sw = actuators_desired.motor_left_sw;
-// global_data_unlock(&global_data_ardrone_control->access_conf);
-// global_data_broadcast(&global_data_ardrone_control->access_conf);
- //END DEBUG
-
-
-
-// gettimeofday(&tv, NULL);
-// now = ((uint32_t)tv.tv_sec) * 1000 + tv.tv_usec/1000;
-// time_elapsed = now - last_run;
-// if (time_elapsed*1000 > CONTROL_LOOP_USLEEP)
-// {
-// sleep_time = (int32_t)CONTROL_LOOP_USLEEP - ((int32_t)time_elapsed*1000 - (int32_t)CONTROL_LOOP_USLEEP);
-//
-// if(motor_skip_counter %500 == 0)
-// {
-// printf("Desired: %u, New usleep: %i, Time elapsed: %u, Now: %u, Last run: %u\n",(uint32_t)CONTROL_LOOP_USLEEP, sleep_time, time_elapsed*1000, now, last_run);
-// }
-// }
-//
-// if (sleep_time <= 0)
-// {
-// printf("WARNING: CPU Overload!\n");
-// printf("Desired: %u, New usleep: %i, Time elapsed: %u, Now: %u, Last run: %u\n",(uint32_t)CONTROL_LOOP_USLEEP, sleep_time, time_elapsed*1000, now, last_run);
-// usleep(CONTROL_LOOP_USLEEP);
-// }
-// else
-// {
-// usleep(sleep_time);
-// }
-// last_run = now;
-//
-// now_time = hrt_absolute_time();
-// if(control_counter % 500 == 0)
-// {
-// printf("Now: %lu\n",(unsigned long)now_time);
-// printf("Last: %lu\n",(unsigned long)last_time);
-// printf("Difference: %lu\n", (unsigned long)(now_time - last_time));
-// printf("now seconds: %lu\n", (unsigned long)(now_time / 1000000));
-// }
-// last_time = now_time;
-//
-// now_time = hrt_absolute_time() / 1000000;
-// if(now_time - last_time > 0)
-// {
-// printf("Counter: %ld\n",control_counter);
-// last_time = now_time;
-// control_counter = 0;
-// }
-// control_counter++;
-}
diff --git a/apps/sensors/sensors.c b/apps/sensors/sensors.c
index 564ee4f8c..77b7b9c47 100644
--- a/apps/sensors/sensors.c
+++ b/apps/sensors/sensors.c
@@ -966,7 +966,7 @@ int sensors_main(int argc, char *argv[])
if (!thread_running) {
printf("sensors app not started\n");
} else {
- printf("stopping sensors app");
+ printf("stopping sensors app\n");
thread_should_exit = true;
}
exit(0);
diff --git a/apps/multirotor_control/pid.c b/apps/systemlib/pid/pid.c
index 5ce05670e..a24657a41 100644
--- a/apps/multirotor_control/pid.c
+++ b/apps/systemlib/pid/pid.c
@@ -1,6 +1,45 @@
-#include "pid.h"
+/****************************************************************************
+ *
+ * Copyright (C) 2008-2012 PX4 Development Team. All rights reserved.
+ * Author: @author Laurens Mackay <mackayl@student.ethz.ch>
+ * @author Tobias Naegeli <naegelit@student.ethz.ch>
+ * @author Martin Rutschmann <rutmarti@student.ethz.ch>
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ * notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in
+ * the documentation and/or other materials provided with the
+ * distribution.
+ * 3. Neither the name PX4 nor the names of its contributors may be
+ * used to endorse or promote products derived from this software
+ * without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
+ * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
+ * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+ * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ ****************************************************************************/
-#include <px4/attitude_estimator_bm/matrix.h> //TODO: move matrix.h to somewhere else?
+/**
+ * @file pid.c
+ * Implementation of generic PID control interface
+ */
+
+#include "pid.h"
void pid_init(PID_t *pid, float kp, float ki, float kd, float intmax,
uint8_t mode, uint8_t plot_i)
diff --git a/nuttx/configs/px4fmu/nsh/appconfig b/nuttx/configs/px4fmu/nsh/appconfig
index 4f0c5093f..01b67cfef 100644
--- a/nuttx/configs/px4fmu/nsh/appconfig
+++ b/nuttx/configs/px4fmu/nsh/appconfig
@@ -56,6 +56,10 @@ CONFIGURED_APPS += systemcmds/mixer
# https://pixhawk.ethz.ch/px4/dev/hello_sky
# CONFIGURED_APPS += examples/px4_simple_app
+# Tutorial code from
+# https://pixhawk.ethz.ch/px4/dev/deamon
+CONFIGURED_APPS += examples/px4_deamon_app
+
CONFIGURED_APPS += uORB
ifeq ($(CONFIG_MAVLINK),y)
diff --git a/apps/multirotor_position_control/rate_control.h b/apps/systemlib/pid/pid.h
index a69745d9b..d62843ed4 100644
--- a/apps/multirotor_position_control/rate_control.h
+++ b/apps/systemlib/pid/pid.h
@@ -1,8 +1,9 @@
/****************************************************************************
*
* Copyright (C) 2008-2012 PX4 Development Team. All rights reserved.
- * Author: Tobias Naegeli <nagelit@student.ethz.ch>
- * Lorenz Meier <lm@inf.ethz.ch>
+ * Author: @author Laurens Mackay <mackayl@student.ethz.ch>
+ * @author Tobias Naegeli <naegelit@student.ethz.ch>
+ * @author Martin Rutschmann <rutmarti@student.ethz.ch>
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
@@ -33,16 +34,41 @@
*
****************************************************************************/
/*
* @file Definition of attitude rate control
*/
-#ifndef RATE_CONTROL_H_
-#define RATE_CONTROL_H_
-#include <uORB/uORB.h>
-#include <uORB/topics/ardrone_motors_setpoint.h>
-#include <uORB/topics/sensor_combined.h>
+#ifndef PID_H_
+#define PID_H_
-void control_rates(struct sensor_combined_s *raw, struct ardrone_motors_setpoint_s *setpoints);
+#include <stdint.h>
-#endif /* RATE_CONTROL_H_ */
+/* PID_MODE_DERIVATIV_CALC calculates discrete derivative from previous error
+ * val_dot in pid_calculate() will be ignored */
+#define PID_MODE_DERIVATIV_CALC 0
+/* Use PID_MODE_DERIVATIV_SET if you have the derivative already (Gyros, Kalman) */
+#define PID_MODE_DERIVATIV_SET 1
+
+typedef struct {
+ float kp;
+ float ki;
+ float kd;
+ float intmax;
+ float sp;
+ float integral;
+ float error_previous;
+ uint8_t mode;
+ uint8_t plot_i;
+ uint8_t count;
+ uint8_t saturated;
+} PID_t;
+
+void pid_init(PID_t *pid, float kp, float ki, float kd, float intmax, uint8_t mode, uint8_t plot_i);
+void pid_set_parameters(PID_t *pid, float kp, float ki, float kd, float intmax);
+//void pid_set(PID_t *pid, float sp);
+float pid_calculate(PID_t *pid, float sp, float val, float val_dot, float dt);
+
+
+
+#endif /* PID_H_ */
@@ -33,16 +34,41 @@
*
****************************************************************************/
-/*
- * @file Definition of attitude rate control
+/**
+ * @file pid.h
+ * Definition of generic PID control interface
*/